// mozileSourceEdit.js

/* ***** BEGIN LICENSE BLOCK *****
 * Licensed under Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * Full Terms at http://mozile.mozdev.org/license.html
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Playsophy code.
 *
 * The Initial Developer of the Original Code is Playsophy
 * Portions created by the Initial Developer are Copyright (C) 2002-2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *		James A. Overton <james@overton.ca>
 *
 * ***** END LICENSE BLOCK ***** */

/*******

Mozile Source Edit Javascript
Mozile version 0.6.19

Created by James A. Overton
Created on 2004-02-18

Last revised by James A. Overton
Last revised on 2005-02-18

HISTORY:
	This code has been moved from the mozileUtilities.js file.


DESCRIPTION:
	These functions handle the behaviour of the Source Edit box.

*********/


var mozileLastEA = "";

// The following function concern the source editing features.

function mozileSourceInput() {
	if(!mozileSourceSync || mozileSourceSync == "") {
		mozileSourceSync = document.getElementById("mozileSourceSync");
	}
	if(mozileSourceSync && mozileSourceSync.getAttribute("checked")=="true") {
		mozileUpdateMarkupView();
	}
}

function mozileMarkupInput() {
	if(!mozileSourceSync || mozileSourceSync=="") {
		mozileSourceSync = document.getElementById("mozileSourceSync");
	}
	if(mozileSourceSync && mozileSourceSync.getAttribute("checked")=="true") {
		mozileUpdateSourceView();
	}
}

function mozileSourceModeToggle() {
	var msg1 = "Edit Page Source";
	var msg2 = "Edit Block Source";
	var toggle = document.getElementById("mozileSourceModeToggleButton");
	
	if(toggle.getAttribute("label") == msg1) {
		mozileSourceMode = 0;
		toggle.setAttribute("label", msg2);
	}
	else {
		mozileSourceMode = 1;
		toggle.setAttribute("label", msg1);
	}
	mozileUpdateSourceView();
}

function replaceAll( str, from, to ) {
  var idx = str.indexOf( from );
  while ( idx > -1 ) {
    str = str.replace( from, to ); 
    idx = str.indexOf( from );
  }
  return str;
}

	// pop up a window with the details of the last XML parser error
function mozileSourceShowError() {
	var errorWindow = window.open("","mozileErrorWindow", " ");
	
	var parsererror = mozileSourceError.getElementsByTagName('parsererror')[0].firstChild.nodeValue;
	
	var sourcetext = mozileSourceError.getElementsByTagName('sourcetext')[0].firstChild.nodeValue;
	sourcetext = replaceAll(sourcetext,'<','&lt;');
	sourcetext = replaceAll(sourcetext,'>','&gt;');
	
	errorWindow.document.open();
	errorWindow.document.write( '<h4 style="white-space: pre">'+parsererror+'</h4>\n<code style="white-space: pre">'+sourcetext+'</code>' );
	errorWindow.document.close();
}



function mozileUpdateSourceView() {
	var f = ["mozileSourceEdit.js","mozileUpdateSourceView"];
	mozileDebug(f,3,"Source is being updated...");

	if(!mozileSourceBox || mozileSourceBox==""){
		mozileSourceBox = document.getElementById("mozileSourceBox");
	}	
		// if the source box doesn't exist or is not visible, don't bother updating.
	if(!mozileSourceBox || mozileSourceBox.style.visibility=="collapse") return;
	
	if(!mozileSourceTextbox || mozileSourceTextbox ==""){
		mozileSourceTextbox = document.getElementById("mozileSourceTextbox");
	}	
	
	try {
		if(mozileTarget) {
			var editableArea="";
			try {
					// If the source mode is "1" then edit the current block.
				if(mozileSourceMode) {
					var cssr = mozileSelection.getEditableRange();
					editableArea = cssr.top;
				}
					// else edit the whole page
				else {
					editableArea = mozileTarget.document.documentElement;
				}
				mozileLastEA=editableArea;
				mozileDebug(f,2,"Editable Area: "+editableArea);
			} catch(e) {
				mozileDebug(f,2,"No Editable Area: "+e);
				editableArea=mozileLastEA;
			}
				// Decide whether to use HTML or XML mode
			if(mozileTarget.document.contentType=="text/html") {
				mozileSourceTextbox.value = mozileGetInnerHTML(editableArea);
			}
			else {
				if(mozileSourceNamespace == "") {
					mozileSourceNamespace = document.getElementById("mozileSourceNamespace");
				}
				if(mozileSourceNamespace.getAttribute("checked") == "true") {
					mozileSourceTextbox.value = mozileSerializeCurrentNS(editableArea);
				}
				else {
					mozileSourceTextbox.value = mozileSerializeCurrent(editableArea);
				}
			}
		
		}
		else {
			mozileSourceTextbox.value = "No editable selection.";
		}
	} catch(e) {
		mozileDebug(f,2,"Error getting innerHTML: "+e);
	}
}


function mozileUpdateMarkupView() {
	var f = ["mozileSourceEdit.js","mozileUpdateMarkupView"];
	mozileDebug(f,3,"Markup is being updated...");
	
	if(!mozileSourceTextbox || mozileSourceTextbox ==""){
		mozileSourceTextbox = document.getElementById("mozileSourceTextbox");
	}	
	
	try {
		var editableArea="";
		try {
					// If the source mode is "1" then edit the current block.
				if(mozileSourceMode) {
					var cssr = mozileSelection.getEditableRange();
					editableArea = cssr.top;
				}
					// else edit the whole page
				else {
					editableArea = mozileTarget.document.documentElement;
				}
			mozileLastEA=editableArea;
			mozileDebug(f,2,"Editable Area: "+editableArea);
		} catch(e) {
			mozileDebug(f,2,"No Editable Area: "+e);
			editableArea=mozileLastEA;
		}
			// Decide whether to use HTML or XML mode
		if(mozileTarget.document.contentType=="text/html") {
			mozileSetInnerHTML(editableArea, mozileSourceTextbox.value);
		}
		else {
			var result = mozileSetInnerXML(editableArea, mozileSourceTextbox.value);
			mozileDebug(f,2,"Result: "+result);
			if(!result) {
				document.getElementById('mozileSourceError').setAttribute("collapsed","false");
			}
			else {
				document.getElementById('mozileSourceError').setAttribute("collapsed","true");
			}
		}
		//mozileDo("Source Editing");
	} catch(e) {
		mozileDebug(f,2,"Error setting innerXML or innerHTML: "+e);
	}
}

	// toggles the visibility of the source view box
function mozileShowSourceView() {
	var f = ["mozileSourceEdit.js","mozileShowSourceView"];
	mozileDebug(f,3,"Toggling source view...");
	//alert("Show Source!");
	var button = document.getElementById("mozile-edit-source");
	var mozileSourceBox, mozileSourceVbox, mozileSourceSplitter, xmlDoc;
		// First see we have the splitter and box already
	mozileDebug(f,3,"Source tools: "+ mozileSourceSplitter + mozileSourceBox);
	if( !mozileSourceSplitter || !mozileSourceBox ) {
			// If tey aren't defined, then try to grab them
		mozileDebug(f,3,"Detecting source tools");
		mozileSourceSplitter = document.getElementById("mozileSourceSplitter");
		mozileSourceBox = document.getElementById("mozileSourceBox");
		mozileDebug(f,3,"Source tools: "+ mozileSourceSplitter + mozileSourceBox);
			// If they're still not there, create them.
		if( !mozileSourceSplitter || !mozileSourceBox ) {
			mozileDebug(f,3,"Creating source tools");
				// Grab the container
			mozileSourceVbox = document.getElementById("appcontent");
			
			xmlDoc = mozileLoadSourceBox();
			mozileSourceSplitter = xmlDoc.getElementById("mozileSourceSplitter").cloneNode(true);
			mozileSourceBox = xmlDoc.getElementById("mozileSourceBox").cloneNode(true);
			
			mozileDebug(f,3,"Source tools: "+ mozileSourceSplitter + mozileSourceBox);
				// append the nodes
			mozileSourceVbox.appendChild(mozileSourceSplitter);
			mozileSourceVbox.appendChild(mozileSourceBox);
			mozileSourceSplitter.style.visibility = "collapse";
			mozileSourceBox.style.visibility = "collapse";

		}
	}
		// If I use the "collapse" XUL attribute then the textbox won't trigger oninput events, so instead we use the "visibility" CSS property
	if(mozileSourceBox.style.visibility == "collapse") {
		mozileSourceSplitter.style.visibility = "visible";
		mozileSourceBox.style.visibility = "visible";
		button.setAttribute("active","true");
	}
	else {
		mozileSourceSplitter.style.visibility = "collapse";
		mozileSourceBox.style.visibility = "collapse";
		button.setAttribute("active","false");
	}
	
	mozileUpdateSourceView();
}


// Functions to get and set the content of an element

function mozileSetInnerXML(node,str) {
	var f = ["mozileSourceEdit.js","mozileSetInnerXML"];
	mozileDebug(f,3,"Setting inner XML...");
	try {
		var xdoc = mozileStrToDOM(str);
		mozileDebug(f,2,"XDOC: "+ xdoc +" => "+ printXML(xdoc));
		if(xdoc.documentElement.nodeName == "parsererror") {
			mozileDebug(f,2,"Parser Error!");
			mozileSourceError = xdoc;
			return false;
		}
		mozileReplaceNodeWithDF(node, xdoc);
	} catch(e) {
		mozileDebug(f,1,"Set Inner XML Error: "+e);
	}
	return true;
}

function mozileGetInnerXML(node) {
	var f = ["mozileSourceEdit.js","mozileGetInnerXML"];
	mozileDebug(f,3,"Getting inner XML...");

	//create a new XMLSerializer
	try {
		var objXMLSerializer = new XMLSerializer;
	} catch(e) {
		mozileDebug(f,0,"Your browser does not seem to be equipped with the required 'XML Extras'. These are standard in the mozilla.org browser releases, but may not be included in some custom releases. Mozile cannot save documents without XMLSerializer. There was an error when trying to create a new XMLSerializer: "+e);
		return false;
	}
	
	var dataToSaveRange = document.createRange();
	dataToSaveRange.selectNodeContents(node);
	var dataToSave = dataToSaveRange.cloneContents();

	var contentToSave = objXMLSerializer.serializeToString(dataToSave);
	
	return contentToSave;
}



function mozileSetInnerHTML(node,str){
	
	if(node==node.ownerDocument.documentElement) {
		var f = ["mozileSourceEdit.js","mozileSetInnerHTML"];
		mozileDebug(f,3,"Trying to set innerHTML for the document element");
		mozileDebug(f,3,"String: "+ str);
		try {
			var doc = node.ownerDocument;
			var head = doc.getElementsByTagName('head')[0];
			var body = doc.getElementsByTagName('body')[0];
			
			mozileRemoveChildren(head);
			mozileRemoveChildren(body);
			//mozileDebug(f,3,"Any left?: "+ head.childNodes.length +":"+body.childNodes.length );
	
			var wrapper = doc.createElement('wrapper');
			wrapper.innerHTML = str;
					
			var children = wrapper.childNodes;
			mozileDebug(f,3,"Length: "+ children.length );
			
				// clone each child node "deep" and insert it before "next"
			var kiddies = "";
			var inHead = 1;
			for(var i=0;i<children.length;i++) {
				var newNode = children[i].cloneNode(true);
				kiddies = kiddies + ", "+ newNode.localName;
				if(inHead) {
					head.appendChild(newNode);
				}
				else {
					body.appendChild(newNode);
				}
				if(newNode.nodeType==1 && newNode.id=="mozIECE.js") {
					inHead = 0;
				}
			}
			mozileDebug(f,3,"Kiddies: "+ kiddies);
		} catch(e) {
			alert(e);
		}
	}
	else {
		node.innerHTML=str;
	}
/*	var r = node.ownerDocument.createRange();
	r.selectNodeContents(node);
	r.deleteContents();
	var df = r.createContextualFragment(str);
	node.appendChild(df);
*/
	return str;
}


function mozileGetInnerHTML(node){
/*
	var str = "";
	for (var i=0; i<node.childNodes.length; i++)
		str += node.ownerDocument.getOuterHTML(node.childNodes.item(i));
	return str;
*/
	return node.innerHTML;
}


	// Load the sourcebox.xul file and return the string of its contents
function mozileLoadSourceBox() {
	var f = ["mozileSourceEdit.js","mozileLoadSourceBox"];
	mozileDebug(f,3,"Load Source Box starting");
	
	var sourceBoxFile = "chrome://mozile/content/XUL/sourcebox.xul";
	var	loaded, xmlDoc;
	
	try {
 		xmlDoc = document.implementation.createDocument("", "", null);
		xmlDoc.async = false;
		loaded = xmlDoc.load(sourceBoxFile);
		if( !loaded ) {
				mozileDebug(f,1,"Source Box file load has failed");
				return false;
		}
	}catch(e){
			mozileDebug(f,1,"Exception: "+e);
	}

	mozileDebug(f,3,"Source Box file loaded");
	return xmlDoc;
}